/****************************************************************************
**
** This file is part of a Qt Solutions component.
** 
** Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
** 
** Contact:  Qt Software Information (qt-info@nokia.com)
** 
** Commercial Usage  
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Solutions Commercial License Agreement provided
** with the Software or, alternatively, in accordance with the terms
** contained in a written agreement between you and Nokia.
** 
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
** 
** In addition, as a special exception, Nokia gives you certain
** additional rights. These rights are described in the Nokia Qt LGPL
** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
** package.
** 
** GNU General Public License Usage 
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
** 
** Please note Third Party Software included with Qt Solutions may impose
** additional restrictions and it is the user's responsibility to ensure
** that they have met the licensing requirements of the GPL, LGPL, or Qt
** Solutions Commercial license and the relevant license of the Third
** Party Software they are using.
** 
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
** 
****************************************************************************/



#ifndef SUBMARINE_P_H
#define SUBMARINE_P_H

//Own
#include "animationmanager.h"
#include "submarine.h"

//Qt
#if defined(QT_EXPERIMENTAL_SOLUTION)
# include "qtanimationstate.h"
# include "qtpropertyanimation.h"
#else
# include <QtAnimationState>
# include <QtPropertyAnimation>
#endif
#include <QGraphicsScene>


//This state is describing when the boat is moving right
class MovementState : public QtAnimationState
{
Q_OBJECT
public:
    MovementState(SubMarine *submarine, QtState *parent = 0) : QtAnimationState(parent)
    {
        movementAnimation = new QtPropertyAnimation(submarine, "pos");
        connect(movementAnimation,SIGNAL(valueChanged(const QVariant &)),this,SLOT(onAnimationMovementValueChanged(const QVariant &)));
        addAnimation(movementAnimation);
        AnimationManager::self()->registerAnimation(movementAnimation);
        this->submarine = submarine;
    }
protected slots:
    void onAnimationMovementValueChanged(const QVariant &)
    {
        if (qrand() % 200 + 1 == 3)
            submarine->launchTorpedo(qrand() % 3 + 1);
    }
protected:
    void onEntry()
    {
        if (submarine->currentDirection() == SubMarine::Left) {
            movementAnimation->setEndValue(QPointF(0,submarine->y()));
            movementAnimation->setDuration(submarine->x()/submarine->currentSpeed()*15);
        }
        else /*if (submarine->currentDirection() == SubMarine::Right)*/ {
            movementAnimation->setEndValue(QPointF(submarine->scene()->width()-submarine->size().width(),submarine->y()));
            movementAnimation->setDuration((submarine->scene()->width()-submarine->size().width()-submarine->x())/submarine->currentSpeed()*15);
        }
        movementAnimation->setStartValue(submarine->pos());
        QtAnimationState::onEntry();
    }

    void onExit()
    {
        movementAnimation->stop();
    }
private:
    SubMarine *submarine;
    QtPropertyAnimation *movementAnimation;
};

//This state is describing when the boat is moving right
class ReturnState : public QtAnimationState
{
public:
    ReturnState(SubMarine *submarine, QtState *parent = 0) : QtAnimationState(parent)
    {
        returnAnimation = new QtPropertyAnimation(submarine, "yRotation");
        AnimationManager::self()->registerAnimation(returnAnimation);
        addAnimation(returnAnimation);
        this->submarine = submarine;
    }
protected:
    void onEntry()
    {
        returnAnimation->stop();
        returnAnimation->setStartValue(submarine->yRotation());
        returnAnimation->setEndValue(submarine->currentDirection() == SubMarine::Right ? 360. : 180.);
        returnAnimation->setDuration(500);
        QtAnimationState::onEntry();
    }

    void onExit()
    {
        returnAnimation->stop();
        submarine->currentDirection() == SubMarine::Right ? submarine->setCurrentDirection(SubMarine::Left) : submarine->setCurrentDirection(SubMarine::Right);
    }
private:
    SubMarine *submarine;
    QtPropertyAnimation *returnAnimation;
};

#endif // SUBMARINE_P_H
